//=================================================================================
//
//					Shape from Shading Project - Basic functions
//					Date: 30-04-2007
//					By Chi-kin CHOW
//					Version 1.0
//
//=================================================================================

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<malloc.h>
#include"..\ACC_Math_Kernel\ACC_Math_Kernel.H"
#include"..\SFS_Structure_Kernel\SFS_Structure_Kernel.H"
#include"..\SfS_Project_Kernel\SfS_Project_Kernel.H"

void SfS_Project_Initialization(SFS_Element *cur_SFS_Element)
{
	//============ Initialize the SFS project
	(*cur_SFS_Element).dsrDepthMap = NULL;
	(*cur_SFS_Element).estDepthMap = NULL;
	(*cur_SFS_Element).Focal_Length = 0.0;
	(*cur_SFS_Element).Grid_X = NULL;
	(*cur_SFS_Element).Grid_Y = NULL;
	(*cur_SFS_Element).Grid_Nx = NULL;
	(*cur_SFS_Element).Grid_Ny = NULL;
	(*cur_SFS_Element).Grid_Nz = NULL;
	(*cur_SFS_Element).IntensityMap = NULL;
	(*cur_SFS_Element).no_Col = 0;
	(*cur_SFS_Element).no_RenderingPmtr = 0;
	(*cur_SFS_Element).no_Row = 0;
	(*cur_SFS_Element).no_SingularPoint = 0;
	(*cur_SFS_Element).OrderMap = NULL;
	(*cur_SFS_Element).Rendering_mode = -1;
	(*cur_SFS_Element).Rendering_Setting = NULL;
	(*cur_SFS_Element).SingularPoint_Set = NULL;

	return;
}

void SfS_Project_Load(char *project_filename, SFS_Element *cur_SFS_Element)
{
	char depthmap_filename[256], intensitymap_filename[256];
	int i, j;
	double bias, invalid_depth;
	FILE *project_ptr;

	SfS_Project_Initialization(cur_SFS_Element);

	project_ptr = fopen(project_filename, "r");

	fscanf(project_ptr, "%s", depthmap_filename);
	SfS_Project_DepthMap_Load(depthmap_filename, &(*cur_SFS_Element).dsrDepthMap, &(*cur_SFS_Element).no_Row, &(*cur_SFS_Element).no_Col);

	fscanf(project_ptr, "%lf", &invalid_depth);
	fscanf(project_ptr, "%lf", &bias);
	for(i=0; i< (*cur_SFS_Element).no_Row; ++i)
		for(j=0; j< (*cur_SFS_Element).no_Col; ++j)
		{
			if((*cur_SFS_Element).dsrDepthMap[i][j] == invalid_depth)
				(*cur_SFS_Element).dsrDepthMap[i][j] = INF;
			else
				(*cur_SFS_Element).dsrDepthMap[i][j] = bias - (*cur_SFS_Element).dsrDepthMap[i][j];
		}

	(*cur_SFS_Element).estDepthMap = malloc(sizeof(double*) * (*cur_SFS_Element).no_Row);
	(*cur_SFS_Element).OrderMap = malloc(sizeof(int*) * (*cur_SFS_Element).no_Row);
	for(i=0; i< (*cur_SFS_Element).no_Row; ++i)
	{
		(*cur_SFS_Element).estDepthMap[i] = malloc(sizeof(double) * (*cur_SFS_Element).no_Col);
		(*cur_SFS_Element).OrderMap[i] = malloc(sizeof(int) * (*cur_SFS_Element).no_Col);
	}

	//============ Load Singular Point Set
	fscanf(project_ptr, "%d", &(*cur_SFS_Element).no_SingularPoint);
	if((*cur_SFS_Element).no_SingularPoint > 0)
	{
		(*cur_SFS_Element).SingularPoint_Set = malloc(sizeof(double*) * (*cur_SFS_Element).no_SingularPoint);
		for(i=0; i< (*cur_SFS_Element).no_SingularPoint; ++i)
		{
			(*cur_SFS_Element).SingularPoint_Set[i] = malloc(sizeof(double) * 3);
			fscanf(project_ptr, "%lf %lf", &(*cur_SFS_Element).SingularPoint_Set[i][0], &(*cur_SFS_Element).SingularPoint_Set[i][1]);
			(*cur_SFS_Element).SingularPoint_Set[i][2] = (*cur_SFS_Element).dsrDepthMap[(int) (*cur_SFS_Element).SingularPoint_Set[i][0]][(int) (*cur_SFS_Element).SingularPoint_Set[i][1]];
		}
	}

	//============ Initialize the Grid
	(*cur_SFS_Element).Grid_X = malloc(sizeof(double*) * (*cur_SFS_Element).no_Row);
	(*cur_SFS_Element).Grid_Y = malloc(sizeof(double*) * (*cur_SFS_Element).no_Row);
	(*cur_SFS_Element).Grid_Nx = malloc(sizeof(double*) * (*cur_SFS_Element).no_Row);
	(*cur_SFS_Element).Grid_Ny = malloc(sizeof(double*) * (*cur_SFS_Element).no_Row);
	(*cur_SFS_Element).Grid_Nz = malloc(sizeof(double*) * (*cur_SFS_Element).no_Row);
	for(i=0; i< (*cur_SFS_Element).no_Row; ++i)
	{
		(*cur_SFS_Element).Grid_X[i] = malloc(sizeof(double) * (*cur_SFS_Element).no_Col);
		(*cur_SFS_Element).Grid_Y[i] = malloc(sizeof(double) * (*cur_SFS_Element).no_Col);
		(*cur_SFS_Element).Grid_Nx[i] = malloc(sizeof(double) * (*cur_SFS_Element).no_Col);
		(*cur_SFS_Element).Grid_Ny[i] = malloc(sizeof(double) * (*cur_SFS_Element).no_Col);
		(*cur_SFS_Element).Grid_Nz[i] = malloc(sizeof(double) * (*cur_SFS_Element).no_Col);
		for(j=0; j< (*cur_SFS_Element).no_Col; ++j)
		{
			(*cur_SFS_Element).Grid_X[i][j] = i - (*cur_SFS_Element).no_Row / 2.0;
			(*cur_SFS_Element).Grid_Y[i][j] = j - (*cur_SFS_Element).no_Col / 2.0;
		}
	}

	fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Focal_Length);

	fscanf(project_ptr, "%d", &(*cur_SFS_Element).Rendering_mode);

	switch ((*cur_SFS_Element).Rendering_mode)
	{
		case 0:		//============ Distinct, Oblique
		case 1:		//============ Distinct, Specular
			(*cur_SFS_Element).no_RenderingPmtr = 2;
			(*cur_SFS_Element).Rendering_Setting = malloc(sizeof(double) * (*cur_SFS_Element).no_RenderingPmtr);

			//[0 - 1] = Light Source Direction
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[0]);
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[1]);
			break;
		case 2:		//============ Close, Oblique
		case 3:		//============ Close, Specular
			(*cur_SFS_Element).no_RenderingPmtr = 3;
			(*cur_SFS_Element).Rendering_Setting = malloc(sizeof(double) * (*cur_SFS_Element).no_RenderingPmtr);

			//[0 - 2] = Light Source Position
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[0]);
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[1]);
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[2]);
			break;
		case 4:		//============ Phong Model
			(*cur_SFS_Element).no_RenderingPmtr = 6;
			(*cur_SFS_Element).Rendering_Setting = malloc(sizeof(double) * (*cur_SFS_Element).no_RenderingPmtr);

			//[0 - 1] = Light Source Direction
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[0]);
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[1]);

			//[2 - 5] = Albedo Parameters
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[2]);
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[3]);
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[4]);
			fscanf(project_ptr, "%lf", &(*cur_SFS_Element).Rendering_Setting[5]);
			break;
		default:
			(*cur_SFS_Element).no_RenderingPmtr = 0;
			break;
	}

	fscanf(project_ptr, "%s", intensitymap_filename);
	if(intensitymap_filename[0] != '-')
	{
		SfS_Project_IntensityMap_Load(intensitymap_filename, &(*cur_SFS_Element).IntensityMap, &(*cur_SFS_Element).no_Row, &(*cur_SFS_Element).no_Col);
		for(i=0; i< (*cur_SFS_Element).no_Row; ++i)
			for(j=0; j< (*cur_SFS_Element).no_Col; ++j)
				if((*cur_SFS_Element).dsrDepthMap[i][j] == INF ||
					(*cur_SFS_Element).IntensityMap[i][j] <= 0.1)
					(*cur_SFS_Element).IntensityMap[i][j] = INF;
	}
	else
	{
		(*cur_SFS_Element).IntensityMap = malloc(sizeof(double*) * (*cur_SFS_Element).no_Row);
		for(i=0; i< (*cur_SFS_Element).no_Row; ++i)
			(*cur_SFS_Element).IntensityMap[i] = malloc(sizeof(double) * (*cur_SFS_Element).no_Col);
	}

	fclose(project_ptr);

	return;
}

void SfS_Project_Information(SFS_Element cur_SFS_Element)
{
	int i;

	printf("No. of row = %d, No. of col = %d\n", cur_SFS_Element.no_Row, cur_SFS_Element.no_Col);
	printf("Focal length = %lf\n", cur_SFS_Element.Focal_Length);
	printf("Rendering mode = %d | No. of Rendering Parameter = %d\n", cur_SFS_Element.Rendering_mode, cur_SFS_Element.no_RenderingPmtr);
	printf("Rendering parameters :\n");
	for(i=0; i< cur_SFS_Element.no_RenderingPmtr; ++i)
		printf("%d - %1.3lf\n", i, cur_SFS_Element.Rendering_Setting[i]);
	printf("No. of Singular Point = %d:\n", cur_SFS_Element.no_SingularPoint);
	for(i=0; i< cur_SFS_Element.no_SingularPoint; ++i)
		printf("%d: %d %d %lf\n", i, (int) cur_SFS_Element.SingularPoint_Set[i][0], (int) cur_SFS_Element.SingularPoint_Set[i][1], cur_SFS_Element.SingularPoint_Set[i][2]);
//		printf("%d: %lf %lf %lf\n", i, cur_SFS_Element.Grid_X[(int) cur_SFS_Element.SingularPoint_Set[i][0]][(int) cur_SFS_Element.SingularPoint_Set[i][1]],
//									 cur_SFS_Element.Grid_Y[(int) cur_SFS_Element.SingularPoint_Set[i][0]][(int) cur_SFS_Element.SingularPoint_Set[i][1]],
//									 cur_SFS_Element.SingularPoint_Set[i][2]);

	return;
}

void SfS_Project_Duplication(SFS_Element *cur_SFS_Element, SFS_Element *tgr_SFS_Element)
{
	int i, j;

	SfS_Project_Construction(tgr_SFS_Element, (*cur_SFS_Element).no_Row, (*cur_SFS_Element).no_Col);

	(*tgr_SFS_Element).Focal_Length = (*cur_SFS_Element).Focal_Length;
	(*tgr_SFS_Element).Rendering_mode = (*cur_SFS_Element).Rendering_mode;
	(*tgr_SFS_Element).no_RenderingPmtr = (*cur_SFS_Element).no_RenderingPmtr;
	(*tgr_SFS_Element).Rendering_Setting = malloc(sizeof(double) * (*tgr_SFS_Element).no_RenderingPmtr);
	for(i=0; i< (*cur_SFS_Element).no_RenderingPmtr; ++i)
		(*tgr_SFS_Element).Rendering_Setting[i] = (*cur_SFS_Element).Rendering_Setting[i];

	(*tgr_SFS_Element).no_SingularPoint = (*cur_SFS_Element).no_SingularPoint;
	(*tgr_SFS_Element).SingularPoint_Set = malloc(sizeof(double*) * (*tgr_SFS_Element).no_SingularPoint);
	for(i=0; i< (*tgr_SFS_Element).no_SingularPoint; ++i)
	{
		(*tgr_SFS_Element).SingularPoint_Set[i] = malloc(sizeof(double) * 3);
		for(j=0; j<3; ++j)
			(*tgr_SFS_Element).SingularPoint_Set[i][j] = (*cur_SFS_Element).SingularPoint_Set[i][j];
	}

	for(i=0; i< (*cur_SFS_Element).no_Row; ++i)
		for(j=0; j< (*cur_SFS_Element).no_Col; ++j)
		{
			(*tgr_SFS_Element).Grid_X[i][j] = (*cur_SFS_Element).Grid_X[i][j];
			(*tgr_SFS_Element).Grid_Y[i][j] = (*cur_SFS_Element).Grid_Y[i][j];
			(*tgr_SFS_Element).dsrDepthMap[i][j] = (*cur_SFS_Element).dsrDepthMap[i][j];
			(*tgr_SFS_Element).estDepthMap[i][j] = (*cur_SFS_Element).estDepthMap[i][j];
			(*tgr_SFS_Element).Grid_Nx[i][j] = (*cur_SFS_Element).Grid_Nx[i][j];
			(*tgr_SFS_Element).Grid_Ny[i][j] = (*cur_SFS_Element).Grid_Ny[i][j];
			(*tgr_SFS_Element).Grid_Nz[i][j] = (*cur_SFS_Element).Grid_Nz[i][j];
			(*tgr_SFS_Element).IntensityMap[i][j] = (*cur_SFS_Element).IntensityMap[i][j];
			(*tgr_SFS_Element).OrderMap[i][j] = (*cur_SFS_Element).OrderMap[i][j];
		}

	return;
}

void SfS_Project_Construction(SFS_Element *cur_SFS_Element, int no_row, int no_col)
{
	int i;

	SfS_Project_Initialization(cur_SFS_Element);

	(*cur_SFS_Element).no_Row = no_row;
	(*cur_SFS_Element).no_Col = no_col;

	(*cur_SFS_Element).Grid_X = malloc(sizeof(double*) * no_row);
	(*cur_SFS_Element).Grid_Y = malloc(sizeof(double*) * no_row);
	(*cur_SFS_Element).Grid_Nx = malloc(sizeof(double*) * no_row);
	(*cur_SFS_Element).Grid_Ny = malloc(sizeof(double*) * no_row);
	(*cur_SFS_Element).Grid_Nz = malloc(sizeof(double*) * no_row);
	(*cur_SFS_Element).dsrDepthMap = malloc(sizeof(double*) * no_row);
	(*cur_SFS_Element).estDepthMap = malloc(sizeof(double*) * no_row);
	(*cur_SFS_Element).IntensityMap = malloc(sizeof(double*) * no_row);
	(*cur_SFS_Element).OrderMap = malloc(sizeof(double*) * no_row);

	for(i=0; i< no_row; ++i)
	{
		(*cur_SFS_Element).Grid_X[i] = malloc(sizeof(double) * no_col);
		(*cur_SFS_Element).Grid_Y[i] = malloc(sizeof(double) * no_col);
		(*cur_SFS_Element).Grid_Nx[i] = malloc(sizeof(double) * no_col);
		(*cur_SFS_Element).Grid_Ny[i] = malloc(sizeof(double) * no_col);
		(*cur_SFS_Element).Grid_Nz[i] = malloc(sizeof(double) * no_col);
		(*cur_SFS_Element).dsrDepthMap[i] = malloc(sizeof(double) * no_col);
		(*cur_SFS_Element).estDepthMap[i] = malloc(sizeof(double) * no_col);
		(*cur_SFS_Element).IntensityMap[i] = malloc(sizeof(double) * no_col);
		(*cur_SFS_Element).OrderMap[i] = malloc(sizeof(double) * no_col);
	}

	return;
}

void SfS_Project_Destruction(SFS_Element cur_SFS_Element)
{
	int i;

	free(cur_SFS_Element.Rendering_Setting);

	if(cur_SFS_Element.Grid_X != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.Grid_X[i]);
		free(cur_SFS_Element.Grid_X);
	}

	if(cur_SFS_Element.Grid_Y != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.Grid_Y[i]);
		free(cur_SFS_Element.Grid_Y);
	}

	if(cur_SFS_Element.Grid_Nx != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.Grid_Nx[i]);
		free(cur_SFS_Element.Grid_Nx);
	}

	if(cur_SFS_Element.Grid_Ny != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.Grid_Ny[i]);
		free(cur_SFS_Element.Grid_Ny);
	}

	if(cur_SFS_Element.Grid_Nz != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.Grid_Nz[i]);
		free(cur_SFS_Element.Grid_Nz);
	}

	if(cur_SFS_Element.dsrDepthMap != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.dsrDepthMap[i]);
		free(cur_SFS_Element.dsrDepthMap);
	}

	if(cur_SFS_Element.estDepthMap != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.estDepthMap[i]);
		free(cur_SFS_Element.estDepthMap);
	}

	if(cur_SFS_Element.IntensityMap != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.IntensityMap[i]);
		free(cur_SFS_Element.IntensityMap);
	}

	if(cur_SFS_Element.OrderMap != NULL)
	{
		for(i=0; i< cur_SFS_Element.no_Row; ++i)
			free(cur_SFS_Element.OrderMap[i]);
		free(cur_SFS_Element.OrderMap);
	}

	for(i=0; i< cur_SFS_Element.no_SingularPoint; ++i)
		free(cur_SFS_Element.SingularPoint_Set[i]);
	free(cur_SFS_Element.SingularPoint_Set);

	return;
}

void SingularPoint_Determination(SFS_Element *cur_SFS_Element, int active_range)
{
	int i, j, r, c, active_length, mode;
	int min_row, max_row, min_col, max_col;
	double **cur_SingularPoint_list;

	if((*cur_SFS_Element).no_SingularPoint > 0)
	{
		for(i=0; i< (*cur_SFS_Element).no_SingularPoint; ++i)
			free((*cur_SFS_Element).SingularPoint_Set[i]);
		free((*cur_SFS_Element).SingularPoint_Set);
	}

	active_length = (int) ((active_range - 1) / 2);
	cur_SingularPoint_list = malloc(sizeof(double*) * (*cur_SFS_Element).no_Row * (*cur_SFS_Element).no_Col);
	for(i=0; i< (*cur_SFS_Element).no_Row * (*cur_SFS_Element).no_Col; ++i)
		cur_SingularPoint_list[i] = malloc(sizeof(double) * 3);

	(*cur_SFS_Element).no_SingularPoint = 0; 
	for(i=0; i < (*cur_SFS_Element).no_Row; ++i)
		for(j=0; j < (*cur_SFS_Element).no_Col; ++j)
			if((*cur_SFS_Element).IntensityMap[i][j] != INF)
			{
				mode = 1;

				if(i - active_length < 0)
					min_row = 0;
				else
					min_row = i - active_length;

				if(i + active_length > (*cur_SFS_Element).no_Row - 1)
					max_row = (*cur_SFS_Element).no_Row - 1;
				else
					max_row = i + active_length;

				if(j - active_length < 0)
					min_col = 0;
				else
					min_col = j - active_length;

				if(j + active_length > (*cur_SFS_Element).no_Col - 1)
					max_col = (*cur_SFS_Element).no_Col - 1;
				else
					max_col = j + active_length;

				for(r=min_row; r <= max_row && mode == 1; ++r)
					for(c=min_col; c <= max_col && mode == 1; ++c)
						if((*cur_SFS_Element).IntensityMap[r][c] != INF && (*cur_SFS_Element).IntensityMap[r][c] > (*cur_SFS_Element).IntensityMap[i][j])
							mode = 0;

				if(mode == 1)
				{
					cur_SingularPoint_list[(*cur_SFS_Element).no_SingularPoint][0] = i;
					cur_SingularPoint_list[(*cur_SFS_Element).no_SingularPoint][1] = j;
					cur_SingularPoint_list[(*cur_SFS_Element).no_SingularPoint][2] = (*cur_SFS_Element).dsrDepthMap[i][j];
					++(*cur_SFS_Element).no_SingularPoint; 
				}
			}

	(*cur_SFS_Element).SingularPoint_Set = malloc(sizeof(double*) * (*cur_SFS_Element).no_SingularPoint);
	for(i=0; i< (*cur_SFS_Element).no_SingularPoint; ++i)
	{
		(*cur_SFS_Element).SingularPoint_Set[i] = malloc(sizeof(double) * 3);
		for(j=0; j<3; ++j)
			(*cur_SFS_Element).SingularPoint_Set[i][j] = cur_SingularPoint_list[i][j];
	}

	for(i=0; i< (*cur_SFS_Element).no_Row * (*cur_SFS_Element).no_Col; ++i)
		free(cur_SingularPoint_list[i]);
	free(cur_SingularPoint_list);

	return;
}

double NonBoundaryError(SFS_Element cur_Model, int boundary_width)
{
	int i, j, r, c, boundary_length, no_error;
	double valid, *error_list, error_median;

	no_error = 0;
	error_list = malloc(sizeof(double) * cur_Model.no_Row * cur_Model.no_Col);

	boundary_length = (int) (boundary_width / 2);
	for(i=boundary_length; i < cur_Model.no_Row - boundary_length; ++i)
		for(j=boundary_length; j < cur_Model.no_Col - boundary_length; ++j)
		{
			valid = cur_Model.dsrDepthMap[i][j];
			for(r=-1*boundary_length; r <=boundary_length; ++r)
				for(c=-1*boundary_length; c <=boundary_length; ++c)
				{
					if(cur_Model.dsrDepthMap[i+r][j+c] > valid)
						valid = cur_Model.dsrDepthMap[i+r][j+c];
					if(cur_Model.estDepthMap[i+r][j+c] > valid)
						valid = cur_Model.estDepthMap[i+r][j+c];
				}

			if(valid != INF)
			{
				error_list[no_error] = fabs(cur_Model.dsrDepthMap[i][j] - cur_Model.estDepthMap[i][j]);
				++no_error;
			}
		}

	if(no_error == 0)
		error_median = 0.0;
	else
	{
		Sorting_UnIndexed(&error_list, no_error);
		error_median = error_list[(int) (no_error / 2)];
	}

	free(error_list);

	return error_median;
}

void SfS_Project_DepthMap_Load(char *depthmap_filename, double ***depthmap, int *no_row, int *no_col)
{
	int i, j;
	FILE *depthmap_ptr;

	depthmap_ptr = fopen(depthmap_filename, "r");

	fscanf(depthmap_ptr, "%d", no_row);
	fscanf(depthmap_ptr, "%d", no_col);

	(*depthmap) = malloc(sizeof(double*) * *no_row);
	for(i=0; i< *no_row; ++i)
		(*depthmap)[i] = malloc(sizeof(double) * *no_col);

	for(i=0; i< *no_row; ++i)
		for(j=0; j< *no_col; ++j)
			fscanf(depthmap_ptr, "%lf", &((*depthmap)[i][j]));

	fclose(depthmap_ptr);

	return;
}

void SfS_Project_DepthMap_Write(char *depthmap_filename, double **depthmap, int no_row, int no_col)
{
	int i, j;
	FILE *depthmap_ptr;

	depthmap_ptr = fopen(depthmap_filename, "w");

	fprintf(depthmap_ptr, "%d %d\n", no_row, no_col);

	for(i=0; i< no_row; ++i)
	{
		for(j=0; j< no_col; ++j)
			fprintf(depthmap_ptr, "%lf ", depthmap[i][j]);
		fprintf(depthmap_ptr, "\n");
	}

	fclose(depthmap_ptr);

	return;
}

void SfS_Project_SingularPoint_Write(char *singularpt_filename, SFS_Element cur_Model)
{
	int i, j;
	int row_index, col_index;
	SFS_Element tmp_Model;
	FILE *singularpt_ptr;

	SfS_Project_Construction(&tmp_Model, cur_Model.no_Row, cur_Model.no_Col);

	for(i=0; i< tmp_Model.no_Row; ++i)
		for(j=0; j< tmp_Model.no_Col; ++j)
			tmp_Model.IntensityMap[i][j] = cur_Model.IntensityMap[i][j];

	for(i=0; i< cur_Model.no_SingularPoint; ++i)
	{
		row_index = (int) cur_Model.SingularPoint_Set[i][0];
		col_index = (int) cur_Model.SingularPoint_Set[i][1];

		tmp_Model.IntensityMap[row_index][col_index] = 1.0 - cur_Model.IntensityMap[row_index][col_index];
		if(row_index > 0 && cur_Model.IntensityMap[row_index-1][col_index] != INF)
			tmp_Model.IntensityMap[row_index-1][col_index] = 1.0 - cur_Model.IntensityMap[row_index-1][col_index];

		if(row_index < tmp_Model.no_Row - 1 && cur_Model.IntensityMap[row_index+1][col_index] != INF)
			tmp_Model.IntensityMap[row_index+1][col_index] = 1.0 - cur_Model.IntensityMap[row_index+1][col_index];

		if(col_index > 0 && cur_Model.IntensityMap[row_index][col_index-1] != INF)
			tmp_Model.IntensityMap[row_index][col_index-1] = 1.0 - cur_Model.IntensityMap[row_index][col_index-1];

		if(col_index < tmp_Model.no_Col - 1 && cur_Model.IntensityMap[row_index][col_index+1] != INF)
			tmp_Model.IntensityMap[row_index][col_index+1] = 1.0 - cur_Model.IntensityMap[row_index][col_index+1];
	}

	singularpt_ptr = fopen(singularpt_filename, "w");

	fprintf(singularpt_ptr, "%d %d\n", tmp_Model.no_Row, tmp_Model.no_Col);
	for(i=0; i< tmp_Model.no_Row; ++i)
	{
		for(j=0; j< tmp_Model.no_Col; ++j)
		{
			if(tmp_Model.IntensityMap[i][j] == INF || tmp_Model.IntensityMap[i][j] < 0.0)
				fprintf(singularpt_ptr, "%lf ", INF);
			else
				fprintf(singularpt_ptr, "%1.10lf ", tmp_Model.IntensityMap[i][j]);
		}
		fprintf(singularpt_ptr, "\n");
	}

	fprintf(singularpt_ptr, "%d\n", cur_Model.no_SingularPoint);
	for(i=0; i< cur_Model.no_SingularPoint; ++i)
		fprintf(singularpt_ptr, "%d %d\n", (int) cur_Model.SingularPoint_Set[i][0], (int) cur_Model.SingularPoint_Set[i][1]);

	fclose(singularpt_ptr);

	SfS_Project_Destruction(tmp_Model);

	return;
}

void SfS_Project_GridMap_Write(char *grid_filename, double **grid_x, double **grid_y, double **grid_z, int no_row, int no_col)
{
	int i, j;
	FILE *grid_ptr;

	grid_ptr = fopen(grid_filename, "w");

	fprintf(grid_ptr, "%d %d\n", no_row, no_col);

	for(i=0; i< no_row; ++i)
		for(j=0; j< no_col; ++j)
			fprintf(grid_ptr, "%lf %lf %lf\n", grid_x[i][j], grid_y[i][j], grid_z[i][j]);

	fclose(grid_ptr);

	return;
}

void SfS_Project_IntensityMap_Load(char *intensitymap_filename, double ***intensitymap, int *no_row, int *no_col)
{
	int i, j;
	FILE *intensitymap_ptr;

	intensitymap_ptr = fopen(intensitymap_filename, "r");

	fscanf(intensitymap_ptr, "%d", no_row);
	fscanf(intensitymap_ptr, "%d", no_col);

	(*intensitymap) = malloc(sizeof(double*) * *no_row);
	for(i=0; i< *no_row; ++i)
		(*intensitymap)[i] = malloc(sizeof(double) * *no_col);

	for(i=0; i< *no_row; ++i)
		for(j=0; j< *no_col; ++j)
			fscanf(intensitymap_ptr, "%lf", &((*intensitymap)[i][j]));

	fclose(intensitymap_ptr);

	return;
}

void SfS_Project_IntensityMap_Write(char *intensitymap_filename, double **intensitymap, int no_row, int no_col)
{
	int i, j;
	FILE *intensitymap_ptr;

	intensitymap_ptr = fopen(intensitymap_filename, "w");

	fprintf(intensitymap_ptr, "%d %d\n", no_row, no_col);
	for(i=0; i< no_row; ++i)
	{
		for(j=0; j< no_col; ++j)
		{
			if(intensitymap[i][j] == INF || intensitymap[i][j] < 0.0)
				fprintf(intensitymap_ptr, "%lf ", INF);
			else
				fprintf(intensitymap_ptr, "%1.10lf ", intensitymap[i][j]);
		}
		fprintf(intensitymap_ptr, "\n");
	}

	fclose(intensitymap_ptr);

	return;
}

void SfS_Project_OrderMap_Write(char *ordermap_filename, SFS_Element cur_Model)
{
	int i, j;
	FILE *ordermap_ptr;

	ordermap_ptr = fopen(ordermap_filename, "w");

	fprintf(ordermap_ptr, "%d %d\n", cur_Model.no_Row, cur_Model.no_Col);

	for(i=0; i< cur_Model.no_Row; ++i)
	{
		for(j=0; j< cur_Model.no_Col; ++j)
			fprintf(ordermap_ptr, "%d ", cur_Model.OrderMap[i][j]);
		fprintf(ordermap_ptr, "\n");
	}

	fclose(ordermap_ptr);

	return;
}

void SfS_Project_ErrorMap_Write(char *errormap_filename, SFS_Element cur_Model)
{
	int i, j;
	FILE *errormap_ptr;

	errormap_ptr = fopen(errormap_filename, "w");

	fprintf(errormap_ptr, "%d %d\n", cur_Model.no_Row, cur_Model.no_Col);

	for(i=0; i< cur_Model.no_Row; ++i)
	{
		for(j=0; j< cur_Model.no_Col; ++j)
		{
			if(cur_Model.dsrDepthMap[i][j] == INF || cur_Model.estDepthMap[i][j] == INF)
				fprintf(errormap_ptr, "%1.3lf ", INF);
			else
				fprintf(errormap_ptr, "%1.3lf ", fabs(cur_Model.dsrDepthMap[i][j] - cur_Model.estDepthMap[i][j]));
		}
		fprintf(errormap_ptr, "\n");
	}

	fclose(errormap_ptr);

	return;
}
